namespace xmlns = "http://www.w3c.org/namespace"
namespace xf = "http://xml.apache.org/cocoon/xmlform/2002"
define function size-attribute($appinfo)
{
if ($appinfo/size) then 
   let $size := <sizeelem size="{$appinfo/size/@value}"/>
   return $size/@size
else ()
}
define function element-caption($element)
{
    let $appinfo := $element/annotation/appinfo[@source="pagesmiths"]
    return
    if ($appinfo/caption) then
       <xf:caption>{$appinfo/caption/text()}</xf:caption>
    else
       <xf:caption>{string($element/@name)}</xf:caption>
}
define function form-elements($formType) 
{
    for $element in $formType/sequence/element
    let $appinfo := $element/annotation/appinfo[@source="pagesmiths"]
    let $elementType := document("xmlforms.xsd")/schema/complexType[@name=$element/@type]
    return
      if ($elementType) then
        if ($element/@maxOccurs = "unbounded") then
			<xf:repeat nodeset="{$element/@name}">
			  {element-caption($element)}
			  {form-elements($elementType)}
			</xf:repeat>
        else
          if ($element/@maxOccurs > 1) then
            for $i in (1 to number($element/@maxOccurs))
            return
		      <xf:group ref="{$element/@name}[{$i}]">
		        {element-caption($element)}
	            {form-elements($elementType)}
			  </xf:group>
          else
			<xf:group ref="{$element/@name}">
			  {element-caption($element)}
			  {form-elements($elementType)}
			</xf:group>
      else if ($appinfo/yesorno) then
        <xf:selectOne ref="{$element/@name}" selectUIType="radio">
          {element-caption($element)}
          <xf:item id="yes">
            <xf:caption>Yes</xf:caption>
            <xf:value>true</xf:value>
          </xf:item>
          <xf:item id="no">
            <xf:caption>No</xf:caption>
            <xf:value>false</xf:value>
          </xf:item>
        </xf:selectOne>
      else if ($element/@maxOccurs > 1) then
        <xf:group>
          {element-caption($element)}
          {
           {-- <xf:repeat nodeset="{$element/@name}">
           --   <xf:textbox ref=".">
           --      <xf:violations class="error"/>
           --   </xf:textbox>
           -- </xf:repeat>
           --}
             for $i in (1 to number($element/@maxOccurs))
             return
               if ($appinfo/size) then 
                 <xf:textbox ref="{$element/@name}[{$i}]" size="{$appinfo/size/@value}">
                   <xf:violations class="error"/>
                 </xf:textbox>
               else
                 <xf:textbox ref="{$element/@name}[{$i}]">
                   <xf:violations class="error"/>
                 </xf:textbox>
          }
        </xf:group>
      else if ($appinfo/checkbox) then
        <xf:selectBoolean ref="{$element/@name}">
          {element-caption($element)}
        </xf:selectBoolean>
      else if ($appinfo/choice/@selectUIType) then
        <xf:selectOne ref="{$element/@name}">
          {($appinfo/choice/@selectUIType)}
          {element-caption($element)}
          {
            for $item in ($appinfo/choice/item) 
            return
              <xf:item id="{$item/@name}">
                <xf:caption>{$item/text()}</xf:caption>
                <xf:value>{string($item/@name)}</xf:value>
              </xf:item>
           }
        </xf:selectOne>
      else if ($appinfo/choice) then
        <xf:selectOne ref="{$element/@name}" selectUIType="radio">
          {element-caption($element)}
          {
            for $item in ($appinfo/choice/item) 
            return
              <xf:item id="{$item/@name}">
                <xf:caption>{$item/text()}</xf:caption>
                <xf:value>{string($item/@name)}</xf:value>
              </xf:item>
           }
        </xf:selectOne>
      else if ($element/@name and ($element/@type = "string")) then
        if ($appinfo/size) then 
          <xf:textbox ref="{$element/@name}" size="{$appinfo/size/@value}">
            {element-caption($element)}
            <xf:violations class="error"/>
          </xf:textbox>
        else
          <xf:textbox ref="{$element/@name}">
            {element-caption($element)}
            <xf:violations class="error"/>
          </xf:textbox>
     else
        ()
}
let $form-header := document("web/forms/form-header.xml")
let $po := document("xmlforms.xsd")/schema/complexType[@name="PURCHASEORDERTYPE"]
let $poform :=
<document xmlns:xf="http://xml.apache.org/cocoon/xmlform/2002">
  {$form-header}
  <xf:form id="form-po" view="po" action="forms" ref="PURCHASEORDER">
    <xf:caption>Purchase Order</xf:caption>
    <error>
      <xf:violations class="error"/>
    </error>
    {
      form-elements($po)
    }
    <xf:submit id="view_main" class="button">
      <xf:caption>Main</xf:caption>
    </xf:submit>
    <xf:submit id="view_po_item" class="button">
      <xf:caption>Add Items</xf:caption>
    </xf:submit>
    <xf:submit id="next" class="button">
      <xf:caption>Next</xf:caption>
    </xf:submit>
  </xf:form>
</document>
let $poitem := document("xmlforms.xsd")/schema/complexType[@name="Item"]
let $poitemform :=
<document xmlns:xf="http://xml.apache.org/cocoon/xmlform/2002">
  {$form-header}
  <xf:form id="form-po" view="po_item" action="forms" ref="item">
    <xf:caption>Add Items</xf:caption>
    <error>
      <xf:violations class="error"/>
    </error>
    {
      form-elements($poitem)
    }
    <xf:submit id="prev" class="button">
      <xf:caption>Prev</xf:caption>
    </xf:submit>
    <xf:submit id="add_po_item" class="button">
      <xf:caption>Add Item</xf:caption>
    </xf:submit>
    <xf:submit id="view_po" class="button">
      <xf:caption>Return to PO</xf:caption>
    </xf:submit>
  </xf:form>
</document>
return
( write-to($poform, "build/forms/po.xml")
  , write-to($poitemform, "build/forms/po_item.xml")  )
