package com.pagesmiths.xmlforms;

// Java classes
import java.util.Enumeration;
import java.util.LinkedList;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.FilenameFilter;

// Cocoon 
import org.apache.avalon.framework.parameters.Parameters;

import org.apache.cocoon.acting.AbstractXMLFormAction;

import org.apache.cocoon.environment.Request;

import org.apache.cocoon.components.validation.Violation;

import org.apache.cocoon.components.xmlform.Form;
import org.apache.cocoon.components.xmlform.FormListener;

import org.apache.commons.jxpath.JXPathContext;
import org.apache.commons.jxpath.Pointer;
import org.apache.commons.jxpath.JXPathException;

// Castorized schema classes
import com.pagesmiths.xmlforms.data.*;
import org.exolab.castor.xml.MarshalException;
import org.exolab.castor.xml.ValidationException;

public class FormAction extends AbstractXMLFormAction
   implements FormListener
{
  final static String ROLE_ADMIN = "administrator";
  
  final static String MODEL_SESSION_ATTR = "com-pagesmiths-xmlforms-model";
  
  final static String CLIENTS_DIRECTORY = "clients";
  final static String POS_DIRECTORY = "purchaseorders";
  final static String FILE_EXTENSION = ".xml";
  
  final static FilenameFilter FILE_EXTENSION_FILTER
     = new FilenameFilter()
     {
        public boolean accept (final File dir, final String name)
        {
          return name.endsWith(FILE_EXTENSION);
        }
     };
  
  // different form views
  // participating in the wizard
  final static String VIEW_START = "start";
  final static String VIEW_MAIN = "main";
  final static String VIEW_MAIN_ADMIN = "main_admin";
  final static String VIEW_MAIN_USER = "main_user";
  final static String VIEW_CHOOSE_PO = "choose_po";

  final static String VIEW_PO = "po";
  final static String VIEW_PO_ITEM = "po_item";

  final static Vector adminViews = new Vector();
  final static Vector userViews = new Vector();
  
  static {
     // View sequence for administrators (role=administrator).
     adminViews.add(VIEW_MAIN_ADMIN);
     
     // View sequence for users (role=user).
     userViews.add(VIEW_MAIN_USER);
     userViews.add(VIEW_CHOOSE_PO);
     userViews.add(VIEW_PO);
     userViews.add(VIEW_PO_ITEM);
  }
  
    
  // action commands used in the wizard
  final static String CMD_START = "start";
  final static String CMD_NEXT = "next";
  final static String CMD_PREV = "prev";
  final static String CMD_SAVE = "save";
  final static String CMD_LOAD = "load";
  final static String CMD_ADD_PO_ITEM = "add_po_item";

  final static String CMD_NEW_PO = "new_po";
  final static String CMD_LOAD_PO = "load_po";
  final static String CMD_SAVE_PO = "save_po";

  final static String CMD_VIEW_PREFIX = "view_";

  // HTML Form variable prefix for hidden checkbox fields for view reset.
  final static String CHECKBOX_PREFIX = "xmlform-checkbox$";
  public static String VIOLATION_MESSAGE_BAD_CHECKBOX  
    = "Checkbox reset failed";
  
  protected Vector getViews(final FORMBEAN fb)
  {
     if (ROLE_ADMIN.equals(fb.getUser().getRole())) {
        return adminViews;
     }
     
     return userViews;
  }
  
  protected String getMainView(final FORMBEAN fb)
  {
      return (String) getViews(fb).elementAt(0);
  }
  
  /**
   * Wrap page method so that we can detect when we go to a main
   * view that needs us to update the sponsor or product lists in 
   * the model.
   */
  protected Map page(final String pageName)
  {
    if (VIEW_CHOOSE_PO.equals(pageName)) {
      updatePOList();
      
      final Form form = getForm();
      form.setValue("/listchoice", form.getValue("/PO_ID"));
    }

    if (VIEW_MAIN_USER.equals(pageName)) {
      final Form form = getForm();
      form.setValue("/listchoice", form.getValue("/PO_ID"));
    }

    return super.page(pageName);
  }

  /**
   * The first callback method which is called
   * when an action is invoked.
   *
   * It is called before population.
   *
   *
   * @return null if the Action is prepared to continue - the normal case.
   * an objectModel map which will be immediately returned by the action.
   *
   * This method is a good place to handle buttons with Cancel
   * kind of semantics. For example
   * <pre>if getCommand().equals("Cancel") return page("input");</pre>
   *
   */
  protected Map prepare()
  {
    // following is navigation logic for the GUI version
    if ( getCommand() == null ) {
        // initial link
        return page( VIEW_START );
    }
    
    if (CMD_START.equals(getCommand())) {
      // reset state by removing old form
      // if one exists
      Form.remove( getObjectModel(), getFormId() );
      
      getForm().addFormListener( this );
      
      start();

      return page(getMainView((FORMBEAN) (getForm().getModel())));
    }
    
    if (Form.lookup (getObjectModel(), getFormId()) == null) {
        // session expired
        return page( VIEW_START );
    }

    // get ready for action
    // if not ready return page("whereNext");
    return null;
  }


  /**
   * Invoked after form population
   *
   * Semanticly similar to Struts Action.perform()
   *
   * Take appropriate action based on the command
   *
   */
  public Map perform ()
  {
    // get the actual model which this Form encapsulates
    // and apply additional buziness logic to the model
    // final SponsorBean  jBean = (SponsorBean) getForm().getModel();

    // set the page control flow parameter
    // according to the validation result
    if (getCommand().equals( CMD_NEXT ) && (getForm().getViolations () != null)) {
      // errors, back to the same page
      return page( getFormView() );
    }

      // validation passed
      // continue with control flow

      // clear validation left overs in case the user
      // did not press the Next button
      getForm().clearViolations();

      final FORMBEAN fb = (FORMBEAN) (getForm().getModel());
      
      fb.removeAllMessages();
      
      // get the user submitted command (through a submit button)
      final String command = getCommand();
      // get the form view which was submitted
      final String formView = getFormView();

      if (command.startsWith(CMD_VIEW_PREFIX)) {
         final String cmd_view = command.substring(CMD_VIEW_PREFIX.length());
         
         if (VIEW_MAIN.equals(cmd_view)) {
             return page(getMainView(fb));
         }

         if (getViews(fb).indexOf(cmd_view) >= 0) {
            return page(cmd_view);
         }
         
         // should never reach this statement
         return page( VIEW_START );
      }
       
      if (CMD_NEXT.equals(command)) {
         int view_index = getViews(fb).indexOf(formView);
         
         if (view_index < 0) {
            // should never reach this statement
            return page( VIEW_START );
         }
     
         while ((view_index + 1) <  getViews(fb).size()) {
            final String next_view = (String) getViews(fb).elementAt(view_index + 1);
            
            return page(next_view);
         }
         
         return page(getMainView(fb));
      }
      
      if (CMD_PREV.equals(command)) {
         int view_index = getViews(fb).indexOf(formView);
         
         if (view_index < 0) {
            // should never reach this statement
            return page( VIEW_START );
         }
     
         while (view_index > 0) {
            final String prev_view = (String) getViews(fb).elementAt(view_index - 1);
            
            return page(prev_view);
         }
         
         return page(getMainView(fb));
      }
      
      if (CMD_NEW_PO.equals(command)) {
         newPO(fb);

         return page(formView);
      }         

      if (CMD_SAVE_PO.equals(command)) {
         try {
           fb.setPO_ID(fb.getListchoice());

           fb.getPURCHASEORDER().marshal(new FileWriter(getPOFile(fb)));
           
           {
              final MESSAGE message = MESSAGEHelper.initNew();
              message.setLevel("info");
              message.setText("Puchase Order file saved OK.");
              fb.addMessages(message);
           }
         } catch (final Exception e) {
           getLogger().error("FormAction: CMD_SAVE_SPONSOR failed", e);
           {
              final MESSAGE message = MESSAGEHelper.initNew();
              message.setLevel("error");
              message.setText("ERROR saving Puchase Order file.");
              fb.addMessages(message);
           }
           {
              final MESSAGE message = MESSAGEHelper.initNew();
              message.setLevel("error");
              message.setText(e.getMessage());
              fb.addMessages(message);
           }
         }
          
         return page(formView);
      }         

      if (CMD_LOAD_PO.equals(command)) {
         try {
           fb.setPO_ID(fb.getListchoice());

           final FileReader reader = new FileReader(getPOFile(fb));

           final PURCHASEORDERTYPE data = PURCHASEORDERTYPE.unmarshalPURCHASEORDERTYPE(reader);

           fb.setPURCHASEORDER(data);
             
           setModel(fb);
           
           {
              final MESSAGE message = MESSAGEHelper.initNew();
              message.setLevel("info");
              message.setText("Purchase Order file loaded OK.");
              fb.addMessages(message);
           }
         } catch (final Exception e) {
           getLogger().error("FormAction: CMD_LOAD_SPONSOR failed", e);
           {
              final MESSAGE message = MESSAGEHelper.initNew();
              message.setLevel("error");
              message.setText("ERROR loading Purchase Order file.");
              fb.addMessages(message);
           }
           {
              final MESSAGE message = MESSAGEHelper.initNew();
              message.setLevel("error");
              message.setText(e.getMessage());
              fb.addMessages(message);
           }
         }
           
         return page(getMainView(fb));
      }         

      if (CMD_ADD_PO_ITEM.equals(command)) {
         fb.getPURCHASEORDER().getItems().addItem(fb.getItem());
         fb.setItem(ItemHelper.initNew());
         return page(formView);
      }
      
    // should never reach this statement
    return page( VIEW_START );

  }
  
  public void start()
  {
     getLogger().debug("starting");
     
     loadUserParameters();
     
     // A session attribute so that the Castor transform can get to our data.
     getSession().setAttribute(MODEL_SESSION_ATTR, getForm().getModel());

     getLogger().debug("made it!");

  }
  
  protected void setModel(final FORMBEAN fb)
  {
     getForm().setModel(fb);

     getSession().setAttribute(MODEL_SESSION_ATTR, getForm().getModel());
  }
  
  final protected File getClientDirectory(final FORMBEAN fb, final String directory)
  {
    final String fileName 
       = fb.getUser().getPath()
       + File.separator + CLIENTS_DIRECTORY
       + File.separator + fb.getUser().getClient_id()
       + File.separator + directory; 

     return new File(fileName);
  }

  final protected File getPOFile(final FORMBEAN fb)
  {
     final File clientDirectory = getClientDirectory(fb, POS_DIRECTORY);

     return new File(clientDirectory, fb.getPO_ID() + FILE_EXTENSION);
  }

  public void newPO(final FORMBEAN fb)
  {
     fb.setPURCHASEORDER(PURCHASEORDERTYPEHelper.initNew());
        
     setModel(fb);
  }
    
  public void updatePOList()
  {
     final FORMBEAN fb = (FORMBEAN) (getForm().getModel());
     final File purchaseOrdersDirectory = getClientDirectory(fb, POS_DIRECTORY);
     final File[] purchaseOrderFiles = purchaseOrdersDirectory.listFiles(FILE_EXTENSION_FILTER);
     
     getLogger().debug("updatePOList dir: " + purchaseOrdersDirectory.getPath());
     
     fb.removeAllPURCHASEORDER_INFOLIST();
     
     for (int i = 0; (purchaseOrderFiles != null) && (i < purchaseOrderFiles.length); ++i) {
        final File file = purchaseOrderFiles[i];
        final ITEMINFO info = ITEMINFOHelper.initNew();
        final String filename = file.getName();
        getLogger().debug("updatePOList: " + filename);
        final int extPos = filename.lastIndexOf(FILE_EXTENSION);
        if (extPos > 0) {
           info.setItem_id(filename.substring(0, extPos));
           info.setDescription(filename);
           fb.addPURCHASEORDER_INFOLIST(info);
        }
     }
     
     setModel(fb);
  }
  
  /**
   *
   * FormListener callback
   * called in the beginning Form.populate()
   * before population starts.
   *
   * This is the place to handle unchecked checkboxes.
   *
   */
  public void reset( final Form form )
  {
    final FORMBEAN fb = (FORMBEAN) form.getModel();
    
    // Reset checkboxes
    {    
       final Request request = getRequest();
    
       final Enumeration enum = request.getParameterNames ();
         
       final LinkedList violations = new LinkedList();
         
       while (enum.hasMoreElements ()) {
          final String parameterName = (String) enum.nextElement ();
             
          if (parameterName.startsWith(CHECKBOX_PREFIX)) {
             final Object[] values = request.getParameterValues ( parameterName );
             final String path = parameterName.substring(CHECKBOX_PREFIX.length());
    
             try {
                form.setValue( path, values );
             } catch (final JXPathException  ex) {
                getLogger().debug(ex.toString());
                final Violation v = new Violation();
                v.setPath( path );
                v.setMessage( VIOLATION_MESSAGE_BAD_CHECKBOX );
                violations.add( v );
             }
          }
       } // while

       if (!violations.isEmpty()) {
          form.addViolations(violations);
       }
    }
  }

  /**
   * FormListener callback
   *
   * Invoked during Form.populate();
   *
   * It is invoked before a request parameter is mapped to
   * an attribute of the form model.
   *
   * It is appropriate to use this method for filtering
   * custom request parameters which do not reference
   * the model.
   *
   * Another appropriate use of this method is for graceful filtering of invalid
   * values, in case that knowledge of the system state or
   * other circumstainces make the standard validation
   * insufficient. For example if a registering user choses a username which
   * is already taken - the check requires database transaction, which is
   * beyond the scope of document validating schemas.
   * Of course customized Validators can be implemented to do
   * this kind of domain specific validation
   * instead of using this method.
   *
   *
   * @return false if the request parameter should not be filtered.
   * true otherwise.
   */
  public boolean filterRequestParameter (final Form form, final String parameterName)
  {
    return (parameterName.startsWith(CHECKBOX_PREFIX));
  }

  private final void loadUserParameters()
  {
    final Parameters parameters = getParameters();
    final String userid = parameters.getParameter("com-pagesmiths-xmlforms-user", "UNKNOWN");
    final User user = new User();
    
    UserHelper.init(user);
    
    user.setUser_id(userid);
    user.setName(parameters.getParameter("com-pagesmiths-xmlforms-name", ""));
    user.setRole(parameters.getParameter("com-pagesmiths-xmlforms-role", ""));
    user.setPath(parameters.getParameter("com-pagesmiths-xmlforms-path", ""));
    user.setClient_id(parameters.getParameter("com-pagesmiths-xmlforms-client", ""));
    
    getForm().setValue("/user", user);

    getForm().setValue("/PO_ID", parameters.getParameter("com-pagesmiths-xmlforms-po", ""));

    newPO((FORMBEAN) (getForm().getModel()));
  }

}

