<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet  
    version="1.0"
    xmlns:fo="http://www.w3.org/1999/XSL/Format"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    xmlns:html="http://www.w3.org/1999/xhtml"  >
     
     <xsl:output method="xml"
            version="1.0"
            indent="yes"
            encoding="UTF-8"/>
    <!-- atributes -->
 
    <!-- paper size -->
    <xsl:param name="paper-width">210mm</xsl:param>
    <xsl:param name="paper-height">297mm</xsl:param>
    <xsl:param name="paper-margin-top">15mm</xsl:param>
    <xsl:param name="paper-margin-bottom">20mm</xsl:param>
    <xsl:param name="paper-margin-left">15mm</xsl:param>
    <xsl:param name="paper-margin-right">15mm</xsl:param>

    <!-- header and hooter extension  -->
    <xsl:param name="header-extention-size">13mm</xsl:param>
    <xsl:param name="footer-extention-size">13mm</xsl:param>

    <!-- printing item in header and footer -->
    <xsl:param name="title-print-in-header" select="true()"/>
    <xsl:param name="page-number-print-in-footer" select="true()"/>

    <!-- body -->
    <xsl:param name="body-start-indent">5mm</xsl:param>
    <xsl:param name="body-end-indent">5mm</xsl:param>

   
   <xsl:template   match="html:html">
      <fo:root>
      <xsl:if test="@lang">
      <xsl:attribute name="xml:lang"><xsl:value-of select="@lang" /></xsl:attribute>
      </xsl:if>
      <xsl:if test="@xml:lang">
      <xsl:attribute name="xml:lang"><xsl:value-of select="@xml:lang" /></xsl:attribute>
      </xsl:if>
      <xsl:call-template  name="layout-master-set"/>
      <xsl:apply-templates select="html:body"/>
      </fo:root>
   </xsl:template>
   <!-- Header -->
   <xsl:attribute-set name="h1">
	<xsl:attribute name="start-indent">inherit</xsl:attribute>
	<xsl:attribute name="end-indent">inherit</xsl:attribute>
	<xsl:attribute name="space-before">2pt</xsl:attribute>
	<xsl:attribute name="space-after">2pt</xsl:attribute>
	<xsl:attribute name="font-size">24pt</xsl:attribute>
	<xsl:attribute name="font-weight">bold</xsl:attribute>
	<xsl:attribute name="color">black</xsl:attribute>
    </xsl:attribute-set>
    <xsl:attribute-set name="h2">
	<xsl:attribute name="start-indent">inherit</xsl:attribute>
	<xsl:attribute name="end-indent">inherit</xsl:attribute>
	<xsl:attribute name="space-before">2pt</xsl:attribute>
	<xsl:attribute name="space-after">2pt</xsl:attribute>
	<xsl:attribute name="font-size">16pt</xsl:attribute>
	<xsl:attribute name="font-weight">bold</xsl:attribute>
	<xsl:attribute name="color">black</xsl:attribute>
    </xsl:attribute-set>
    <xsl:attribute-set name="h3">
	<xsl:attribute name="start-indent">body-start()</xsl:attribute>
	<xsl:attribute name="end-indent">inherit</xsl:attribute>
	<xsl:attribute name="space-before">6pt</xsl:attribute>
	<xsl:attribute name="space-after">2pt</xsl:attribute>
	<xsl:attribute name="font-size">14pt</xsl:attribute>
	<xsl:attribute name="font-weight">bold</xsl:attribute>
	<xsl:attribute name="color">black</xsl:attribute>
    </xsl:attribute-set>
    <xsl:attribute-set name="h4">
	<xsl:attribute name="start-indent">inherit</xsl:attribute>
	<xsl:attribute name="end-indent">inherit</xsl:attribute>
	<xsl:attribute name="space-before">1em</xsl:attribute>
	<xsl:attribute name="space-after">0.5em</xsl:attribute>
	<xsl:attribute name="font-size">medium</xsl:attribute>
	<xsl:attribute name="font-weight">bold</xsl:attribute>
	<xsl:attribute name="color">black</xsl:attribute>
    </xsl:attribute-set>
    <xsl:attribute-set name="h5">
	<xsl:attribute name="start-indent">inherit</xsl:attribute>
	<xsl:attribute name="end-indent">inherit</xsl:attribute>
	<xsl:attribute name="space-before">1em</xsl:attribute>
	<xsl:attribute name="space-after">0.5em</xsl:attribute>
	<xsl:attribute name="font-size">small</xsl:attribute>
	<xsl:attribute name="font-weight">bold</xsl:attribute>
	<xsl:attribute name="color">black</xsl:attribute>
    </xsl:attribute-set>
    <xsl:attribute-set name="h6">
	<xsl:attribute name="start-indent">inherit</xsl:attribute>
	<xsl:attribute name="end-indent">inherit</xsl:attribute>
	<xsl:attribute name="space-before">1em</xsl:attribute>
	<xsl:attribute name="space-after">0.5em</xsl:attribute>
	<xsl:attribute name="font-size">x-small</xsl:attribute>
	<xsl:attribute name="font-weight">bold</xsl:attribute>
	<xsl:attribute name="color">black</xsl:attribute>
    </xsl:attribute-set>
   <!-- Block level Attribute -->
    <!-- p-->
   <xsl:attribute-set name="p">
	<xsl:attribute name="font-size">12pt</xsl:attribute>
	<xsl:attribute name="line-height">130%</xsl:attribute>
	<xsl:attribute name="start-indent">5pt</xsl:attribute>
	<xsl:attribute name="end-indent">inherit</xsl:attribute>
<!--	<xsl:attribute name="text-indent">inherit</xsl:attribute>  -->
	<xsl:attribute name="space-before">0.4em</xsl:attribute>
	<xsl:attribute name="space-after">0.4em</xsl:attribute>
    </xsl:attribute-set>
    
    <!-- pre -->
    <xsl:attribute-set name="pre">
        <xsl:attribute name="white-space">pre</xsl:attribute>
        <xsl:attribute name="wrap-option">wrap</xsl:attribute>
        <xsl:attribute name="start-indent">inherit</xsl:attribute>
        <xsl:attribute name="end-indent">inherit</xsl:attribute>
        <xsl:attribute name="text-indent">1em</xsl:attribute>
        <xsl:attribute name="space-before">0.6em</xsl:attribute>
        <xsl:attribute name="space-after">0.6em</xsl:attribute>
     </xsl:attribute-set>
    <!-- pre title-->
    <xsl:attribute-set name="pre-title">
        <xsl:attribute name="start-indent">inherit</xsl:attribute>
        <xsl:attribute name="end-indent">inherit</xsl:attribute>
        <xsl:attribute name="space-before">1em</xsl:attribute>
        <xsl:attribute name="space-after">0.5em</xsl:attribute>
        <xsl:attribute name="font-weight">bold</xsl:attribute>
    </xsl:attribute-set>
        <!-- pre -->
    <xsl:attribute-set name="blockquote">
        <xsl:attribute name="start-indent">inherit + 4em </xsl:attribute>
        <xsl:attribute name="end-indent">inherit + 4em</xsl:attribute>
        <xsl:attribute name="text-indent">1em</xsl:attribute>
        <xsl:attribute name="space-before">0.6em</xsl:attribute>
        <xsl:attribute name="space-after">0.6em</xsl:attribute>
        <xsl:attribute name="margin-top">1em</xsl:attribute>
        <xsl:attribute name="margin-bottom">1em</xsl:attribute>
    </xsl:attribute-set>
    <!-- ul -->
    <!-- for list-block -->
    <xsl:attribute-set name="ul">
<!--        <xsl:attribute name="start-indent">30pt</xsl:attribute> -->
        <xsl:attribute name="provisional-distance-between-starts">1.5em</xsl:attribute>
        <xsl:attribute name="provisional-label-separation">1.5em</xsl:attribute>
    </xsl:attribute-set>
    <xsl:attribute-set name="ul-li">
        <xsl:attribute name="font-size">12pt</xsl:attribute>
        <xsl:attribute name="line-height">130%</xsl:attribute>
        <xsl:attribute name="space-before">0.4em</xsl:attribute>
        <xsl:attribute name="space-after">0.4em</xsl:attribute>
    </xsl:attribute-set>

    <xsl:attribute-set name="ul-li-padding">
    <!--
        <xsl:attribute name="padding-left">0.4em</xsl:attribute>
        <xsl:attribute name="padding-right">0.4em</xsl:attribute>
        <xsl:attribute name="padding-top">0.4em</xsl:attribute>
        <xsl:attribute name="padding-bottom">0.4em</xsl:attribute>
    -->
    </xsl:attribute-set>
    
    
    <!-- ol -->
    <!-- for list-block -->
    <xsl:attribute-set name="ol">
        <xsl:attribute name="provisional-distance-between-starts">1.5em</xsl:attribute>
        <xsl:attribute name="provisional-label-separation">1.5em</xsl:attribute>
    </xsl:attribute-set>
    <xsl:attribute-set name="ol-li">
        <xsl:attribute name="font-size">12pt</xsl:attribute>
        <xsl:attribute name="line-height">130%</xsl:attribute>
        <xsl:attribute name="space-before">0.4em</xsl:attribute>
        <xsl:attribute name="space-after">0.4em</xsl:attribute>
    </xsl:attribute-set>

    <xsl:attribute-set name="ol-li-padding">
    <!--
        <xsl:attribute name="padding-left">3pt</xsl:attribute>
        <xsl:attribute name="padding-right">3pt</xsl:attribute>
        <xsl:attribute name="padding-top">3pt</xsl:attribute>
        <xsl:attribute name="padding-bottom">3pt</xsl:attribute>
    -->
    </xsl:attribute-set>

    <!-- dl dt dd -->
    <xsl:attribute-set name="dl">
        <xsl:attribute name="start-indent">inherit </xsl:attribute>
        <xsl:attribute name="end-indent">inherit </xsl:attribute>
    </xsl:attribute-set>
    <xsl:attribute-set name="dt">
        <xsl:attribute name="start-indent">inherit</xsl:attribute>
        <xsl:attribute name="end-indent">inherit </xsl:attribute>
    </xsl:attribute-set>
    <xsl:attribute-set name="dd">
        <xsl:attribute name="start-indent">inherit +1em</xsl:attribute>
        <xsl:attribute name="end-indent">inherit </xsl:attribute>
        <xsl:attribute name="space-before">0.6em</xsl:attribute>
	<xsl:attribute name="space-after">0.6em</xsl:attribute>
    </xsl:attribute-set>

   <!-- text -->
   <xsl:attribute-set name="em">
 	<xsl:attribute name="font-style">italic</xsl:attribute>
 	<xsl:attribute name="font-weight">bold</xsl:attribute>
     </xsl:attribute-set>
  <xsl:attribute-set name="strong">
  	<xsl:attribute name="font-weight">bold</xsl:attribute>
     </xsl:attribute-set>
  <xsl:attribute-set name="cite">
 	<xsl:attribute name="font-style">italic</xsl:attribute>
 	<xsl:attribute name="font-weight">bold</xsl:attribute>
     </xsl:attribute-set>
  <xsl:attribute-set name="dfn">
 	<xsl:attribute name="font-style">italic</xsl:attribute>
 	<xsl:attribute name="font-weight">bold</xsl:attribute>
     </xsl:attribute-set>
  <xsl:attribute-set name="code">
  </xsl:attribute-set>

  <xsl:attribute-set name="samp">
  </xsl:attribute-set>

  <xsl:attribute-set name="kbd">
  </xsl:attribute-set>

  <xsl:attribute-set name="var">
  	<xsl:attribute name="font-style">italic</xsl:attribute>
 	<xsl:attribute name="font-weight">bold</xsl:attribute>

  </xsl:attribute-set>

  <xsl:attribute-set name="abbr">
  </xsl:attribute-set>

  <xsl:attribute-set name="acronym">
  </xsl:attribute-set>


  <xsl:attribute-set name="sup">
  	<xsl:attribute name="baseline-shift">super</xsl:attribute>
  </xsl:attribute-set>


  <xsl:attribute-set name="sub">
  	<xsl:attribute name="baseline-shift">sub</xsl:attribute>
  </xsl:attribute-set>


  <xsl:attribute-set name="a">
  	<xsl:attribute name="text-decoration">underline</xsl:attribute>
   	<xsl:attribute name="color">blue</xsl:attribute>
  </xsl:attribute-set>

  <xsl:attribute-set name="div">
<!--
  	<xsl:attribute name="start-indent">5mm</xsl:attribute>
   	<xsl:attribute name="end-indent">5mm</xsl:attribute>
-->   	
  </xsl:attribute-set>

   <!-- Head -->
   <xsl:template   match="html:head">
   </xsl:template>

  
   <xsl:template   match="html:title">
      <xsl:apply-templates />
   </xsl:template>

  
   <xsl:template   match="html:base">
      <xsl:apply-templates />
   </xsl:template>

  
   <xsl:template   match="html:meta">
      <xsl:apply-templates />
   </xsl:template>

  
   <xsl:template   match="html:link">
      <xsl:apply-templates />
   </xsl:template>

  
   <xsl:template   match="html:style">
      <xsl:apply-templates />
   </xsl:template>

  
   <xsl:template   match="html:script">
      <xsl:apply-templates />
   </xsl:template>

  
   <xsl:template   match="html:noscript">
      <xsl:apply-templates />
   </xsl:template>
   
   <xsl:template name="layout-master-set">
   <fo:layout-master-set>
       <fo:simple-page-master
                         page-width="{$paper-width}"
                         page-height="{$paper-height}"
                         master-name="BodyPage">
       <fo:region-body   margin-top="{$paper-margin-top}"
                         margin-right="{$paper-margin-right}"
                         margin-bottom="{$paper-margin-bottom}"
                         margin-left="{$paper-margin-left}" />
       <fo:region-before border-after-style="solid"
                         border-width="thin"
                         extent="{$header-extention-size}"
                         display-align="after"/>
       <fo:region-after  border-after-style="solid"
                         border-width="thin"
                         extent="{$footer-extention-size}"
                         display-align="after"/>
       </fo:simple-page-master>
       
       <fo:simple-page-master
                         page-width="{$paper-width}"
                         page-height="{$paper-height}"
                         master-name="CoverPage">
       <fo:region-body   margin-top="{$paper-margin-top}"
                         margin-right="{$paper-margin-right}"
                         margin-bottom="{$paper-margin-bottom}"
                         margin-left="{$paper-margin-left}" />
       </fo:simple-page-master>


        <fo:page-sequence-master master-name="report">
			<fo:repeatable-page-master-alternatives>
				<fo:conditional-page-master-reference
					master-reference="CoverPage"
					page-position="first" />
				<fo:conditional-page-master-reference
					master-reference="BodyPage"
					page-position="rest" />
			</fo:repeatable-page-master-alternatives>
		</fo:page-sequence-master>
		
        <fo:page-sequence-master master-name="plain">
	            <fo:repeatable-page-master-reference
					master-reference="BodyPage"/>
		</fo:page-sequence-master>
   </fo:layout-master-set>
   </xsl:template>
   
   <!-- Body -->
   <xsl:template   match="html:body">
<!--   <fo:page-sequence master-reference="{@master-reference}">  -->
   <fo:page-sequence master-reference="plain">
       <fo:static-content flow-name="xsl-region-before">
           <fo:block text-align="center" font-size="small" space-before="7mm">
               <xsl:if test="$title-print-in-header"><xsl:value-of select="/html:html/html:head/html:title" /></xsl:if>
           </fo:block>
       </fo:static-content>
       <fo:static-content flow-name="xsl-region-after">
           <xsl:if test="$page-number-print-in-footer">
               <fo:block text-align="center" font-size="small">
                    - <fo:page-number/> of <fo:page-number-citation ref-id="last-page"/> -
               </fo:block>
           </xsl:if>
       </fo:static-content>

      <fo:flow flow-name="xsl-region-body" >
        <xsl:apply-templates />
        <fo:block-container id="last-page"/>
      </fo:flow>
   </fo:page-sequence>
   </xsl:template>


   <xsl:template   match="html:div">
      <fo:block xsl:use-attribute-sets="div"><xsl:apply-templates /></fo:block>
   </xsl:template>

  
  <xsl:template match="html:p">
      <fo:block xsl:use-attribute-sets="p">
        <xsl:if test="@id"><xsl:copy-of select="@id"/></xsl:if>
        <xsl:if test="@align"><xsl:attribute name="text-align"><xsl:value-of select="@align"/></xsl:attribute></xsl:if>
        <xsl:apply-templates />
      </fo:block>
  </xsl:template>

   <xsl:template match="html:h1" >
   <fo:block  xsl:use-attribute-sets="h1">
      <xsl:if test="@id"><xsl:copy-of select="@id"/></xsl:if>
      <xsl:if test="@align"><xsl:attribute name="text-align"><xsl:value-of select="@align"/></xsl:attribute></xsl:if>
      <xsl:apply-templates />
   </fo:block>
   </xsl:template>

  
   <xsl:template   match="html:h2" >
   <fo:block xsl:use-attribute-sets="h2">
      <xsl:if test="@id"><xsl:copy-of select="@id"/></xsl:if>
      <xsl:if test="@align"><xsl:attribute name="text-align"><xsl:value-of select="@align"/></xsl:attribute></xsl:if>
      <xsl:apply-templates />
   </fo:block>
   </xsl:template>

  
   <xsl:template   match="html:h3"  >
   <fo:block xsl:use-attribute-sets="h3">
      <xsl:if test="@id"><xsl:copy-of select="@id"/></xsl:if>
      <xsl:if test="@align"><xsl:attribute name="text-align"><xsl:value-of select="@align"/></xsl:attribute></xsl:if>
      <xsl:apply-templates />
   </fo:block>
   </xsl:template>

  
   <xsl:template   match="html:h4" >
   <fo:block xsl:use-attribute-sets="h4">
      <xsl:if test="@id"><xsl:copy-of select="@id"/></xsl:if>
      <xsl:if test="@align"><xsl:attribute name="text-align"><xsl:value-of select="@align"/></xsl:attribute></xsl:if>
      <xsl:apply-templates />
   </fo:block>
   </xsl:template>

   <xsl:template   match="html:h5"  >
   <fo:block xsl:use-attribute-sets="h5">
      <xsl:if test="@id"><xsl:copy-of select="@id"/></xsl:if>
      <xsl:if test="@align"><xsl:attribute name="text-align"><xsl:value-of select="@align"/></xsl:attribute></xsl:if>
      <xsl:apply-templates />
   </fo:block>
   </xsl:template>

  
   <xsl:template   match="html:h6" >
   <fo:block xsl:use-attribute-sets="h6">
      <xsl:if test="@id"><xsl:copy-of select="@id"/></xsl:if>
      <xsl:if test="@align"><xsl:attribute name="text-align"><xsl:value-of select="@align"/></xsl:attribute></xsl:if>
      <xsl:apply-templates />
   </fo:block>
   </xsl:template>

  
   <xsl:template   match="html:ul[html:ul]">
      <xsl:apply-templates/>
   </xsl:template>

   <xsl:template   match="html:ul">
   <fo:block >
       <fo:list-block  xsl:use-attribute-sets="ul">
           <xsl:variable name="level" select="count(ancestor::html:ul)"/>
           <xsl:attribute name="start-indent"><xsl:value-of select='concat(5 + ($level * 19), "pt")'/></xsl:attribute>
           <xsl:apply-templates/>
       </fo:list-block>
   </fo:block>
   </xsl:template>

   <xsl:template   match="html:ul/html:li">
   <fo:list-item xsl:use-attribute-sets="ul-li" >
    <fo:list-item-label end-indent="label-end()">
      <fo:block xsl:use-attribute-sets="ul-li-padding">
          <fo:character character="&#x2022;" />
      </fo:block>
    </fo:list-item-label>
    <fo:list-item-body start-indent="body-start()">
      <fo:block xsl:use-attribute-sets="ul-li-padding">
        <xsl:apply-templates/>
      </fo:block>
    </fo:list-item-body>
   </fo:list-item>
   </xsl:template>
  
   <xsl:template   match="html:ol">
<!--     <xsl:variable name="ol-il-format"><xsl:value-of select="@type"/></xsl:variable> -->
   <fo:block>
       <fo:list-block xsl:use-attribute-sets="ol">
           <xsl:variable name="level" select="count(ancestor::html:ol)"/>
           <xsl:attribute name="start-indent"><xsl:value-of select='concat(5 + ($level * 19), "pt")'/></xsl:attribute>
           <xsl:apply-templates /> 
       </fo:list-block>
   </fo:block>
   </xsl:template>

   <xsl:template   match="html:ol/html:li">
   <fo:list-item xsl:use-attribute-sets="ol-li" >
    <fo:list-item-label end-indent="label-end()">
      <fo:block>
          <xsl:if test="html:h3">
             <xsl:attribute name="space-before">2pt</xsl:attribute>
             <xsl:attribute name="space-after">2pt</xsl:attribute>
             <xsl:attribute name="font-size">14pt</xsl:attribute>
          </xsl:if>
          <xsl:number format="{../@type}." from="html:ol" count="html:li"/>
      </fo:block>
    </fo:list-item-label>
    <fo:list-item-body start-indent="body-start()">
      <fo:block>
        <xsl:apply-templates/>
      </fo:block>
    </fo:list-item-body>
   </fo:list-item>
   </xsl:template>

<!--
   <xsl:template   match="html:ol/html:li/html:h3">
   <fo:list-item>
    <fo:list-item-label xsl:use-attribute-sets="ol-li" end-indent="label-end()">
      <fo:block xsl:use-attribute-sets="ol-li-padding">
          <xsl:number format="{../../@type}." from="html:ol" count="html:li"/>
      </fo:block>
    </fo:list-item-label>
    <fo:list-item-body start-indent="body-start()">
      <fo:block xsl:use-attribute-sets="ol-li-padding">
        <xsl:apply-templates/>
      </fo:block>
    </fo:list-item-body>
   </fo:list-item>
   </xsl:template>
-->

   <xsl:template   match="html:dl">
      <fo:block xsl:use-attribute-sets="dl">
          <xsl:apply-templates />
      </fo:block>
   </xsl:template>

  
   <xsl:template   match="html:dt">
      <fo:block xsl:use-attribute-sets="dt">
          <xsl:apply-templates />
      </fo:block>
   </xsl:template>

  
   <xsl:template   match="html:dd">
      <fo:block xsl:use-attribute-sets="dd">
          <xsl:apply-templates />
      </fo:block>
   </xsl:template>

  
   <xsl:template   match="html:address">
      <xsl:apply-templates />
   </xsl:template>

  
   <xsl:template   match="html:hr">
      <fo:block space-before="1em" space-after="1em">
          <fo:leader leader-pattern="rule" rule-style="groove" leader-length="100%"/>
      </fo:block>
   </xsl:template>

  
   <xsl:template   match="html:pre">
      <xsl:if test="@title">
      <fo:block xsl:use-attribute-sets="pre-title">
          <xsl:value-of select="@title" />
      </fo:block>
      </xsl:if>
      <fo:block xsl:use-attribute-sets="pre">
          <xsl:apply-templates />
      </fo:block>
   </xsl:template>

  
   <xsl:template   match="html:blockquote">
      <fo:block xsl:use-attribute-sets="blockquote">
          <xsl:apply-templates />
      </fo:block>
   </xsl:template>

  
   <xsl:template   match="html:ins">
        <xsl:comment>Inserted By ins tag </xsl:comment>
        <xsl:apply-templates />
   </xsl:template>

  
   <xsl:template   match="html:del">
   <xsl:comment> Deleted By del tag </xsl:comment>
   </xsl:template>

  
   <xsl:template   match="html:a[@href]">
      <fo:inline xsl:use-attribute-sets="a" ><xsl:apply-templates /></fo:inline>
   </xsl:template>

   <xsl:template   match="html:a[@class='page-citation']">
      <fo:page-number-citation ref-id="{@ref-id}"/>
   </xsl:template>

  
   <xsl:template   match="html:span">
      <xsl:apply-templates />
   </xsl:template>

  
   <xsl:template   match="html:bdo">
      <xsl:apply-templates />
      
   </xsl:template>

  
   <xsl:template   match="html:br">
      <fo:block>
        <xsl:if test="@class='page'"><xsl:attribute name="break-after">page</xsl:attribute></xsl:if>
      </fo:block>
   </xsl:template>

  
   <xsl:template   match="html:em">
      <fo:inline xsl:use-attribute-sets="em">
      <xsl:apply-templates />
      </fo:inline>
   </xsl:template>

  
   <xsl:template   match="html:strong">
      <fo:inline xsl:use-attribute-sets="strong">
      <xsl:apply-templates />
      </fo:inline>
   </xsl:template>

  
   <xsl:template   match="html:dfn">
     <fo:inline xsl:use-attribute-sets="dfn"><xsl:apply-templates /></fo:inline>
   </xsl:template>

  
   <xsl:template   match="html:code">
     <fo:inline xsl:use-attribute-sets="code">
     <xsl:apply-templates />
     </fo:inline>
   </xsl:template>

  
   <xsl:template   match="html:samp">
       <fo:inline xsl:use-attribute-sets="samp">
       <xsl:apply-templates />
       </fo:inline>
   </xsl:template>

  
   <xsl:template   match="html:kbd">
       <fo:inline xsl:use-attribute-sets="kbd">
       <xsl:apply-templates />
       </fo:inline>
   </xsl:template>

  
   <xsl:template   match="html:var">
       <fo:inline xsl:use-attribute-sets="var">
       <xsl:apply-templates />
       </fo:inline>
   </xsl:template>

  
   <xsl:template   match="html:cite">
      <fo:inline xsl:use-attribute-sets="cite">
       <xsl:apply-templates />
       </fo:inline>
   </xsl:template>

  
   <xsl:template   match="html:abbr">
       <fo:inline xsl:use-attribute-sets="abbr">
       <xsl:apply-templates />
       </fo:inline>
   </xsl:template>

  
   <xsl:template   match="html:acronym">
       <fo:inline xsl:use-attribute-sets="acronym">
       <xsl:apply-templates />
       </fo:inline>
   </xsl:template>

  
   <xsl:template   match="html:q">
       <xsl:comment>q -- Not Support</xsl:comment>
       <xsl:apply-templates />
   </xsl:template>

  
   <xsl:template   match="html:sub">
       <fo:inline xsl:use-attribute-sets="sub">
       <xsl:apply-templates />
       </fo:inline>
   </xsl:template>

  
   <xsl:template   match="html:sup">
       <fo:inline xsl:use-attribute-sets="sup">
       <xsl:apply-templates />
       </fo:inline>
   </xsl:template>

  
   <xsl:template   match="html:tt">
      <fo:inline font-family="monospace"><xsl:apply-templates /></fo:inline>
   </xsl:template>

  
   <xsl:template   match="html:i">
      <fo:inline font-style="italic"><xsl:apply-templates /></fo:inline>
   </xsl:template>

  
   <xsl:template   match="html:b">
         <fo:inline font-weight="bold"><xsl:apply-templates /></fo:inline>
   </xsl:template>
   
   <xsl:template   match="html:u">
         <fo:inline text-decoration="underline"><xsl:apply-templates /></fo:inline>
   </xsl:template>
   
   <xsl:template   match="html:big">
         <fo:inline font-size="larger"><xsl:apply-templates /></fo:inline>
   </xsl:template>

  
   <xsl:template   match="html:small">
        <fo:inline font-size="smaller"><xsl:apply-templates /></fo:inline>
   </xsl:template>

  
   <xsl:template   match="html:object">
      <xsl:apply-templates />
   </xsl:template>

  
   <xsl:template match="html:param">
      <xsl:apply-templates />
   </xsl:template>

  
   <xsl:template   match="html:html/html:body/html:img">
   <fo:block>
   <fo:external-graphic src="{@src}">
   <xsl:if test="@height">
   <xsl:attribute name="content-height">
     <xsl:value-of select="@height" />px
   </xsl:attribute>
   </xsl:if>
   <xsl:if test="@width">
     <xsl:attribute name="content-width">
       <xsl:value-of select="@width" />px
     </xsl:attribute>
   </xsl:if>
   </fo:external-graphic>
   </fo:block>
   </xsl:template>
   
   <xsl:template   match="html:img">
   <fo:external-graphic src="{@src}">
   <xsl:if test="@height">
   <xsl:attribute name="content-height">
     <xsl:value-of select="@height" />px
   </xsl:attribute>
   </xsl:if>
   <xsl:if test="@width">
     <xsl:attribute name="content-width">
       <xsl:value-of select="@width" />px
     </xsl:attribute>
   </xsl:if>
   </fo:external-graphic>
   </xsl:template>

  
   <xsl:template   match="html:map">
   </xsl:template>

  
   <xsl:template   match="html:area">
   </xsl:template>

  
   <xsl:template   match="html:form">
   </xsl:template>

  
   <xsl:template   match="html:label">
   </xsl:template>

  
   <xsl:template   match="html:input">
   </xsl:template>

  
   <xsl:template   match="html:select">
   </xsl:template>

  
   <xsl:template   match="html:optgroup">
   </xsl:template>

  
   <xsl:template   match="html:option">
   </xsl:template>

  
   <xsl:template   match="html:textarea">
   </xsl:template>

  
   <xsl:template   match="html:fieldset">
    </xsl:template>

  
   <xsl:template   match="html:legend">
   </xsl:template>

  
   <xsl:template   match="html:button">
   </xsl:template>

  
  <xsl:template match="html:table">
<!--
    <fo:table-and-caption>
    <xsl:if test="html:caption">
        <fo:table-caption>
            <xsl:apply-templates select="html:caption"/>
        </fo:table-caption>
   </xsl:if>
-->
   <fo:table>
       <xsl:if test="@width">
       <xsl:attribute name="width"><xsl:value-of select="@width" /></xsl:attribute>
       </xsl:if>
       <xsl:apply-templates select="html:col | html:colgroup"/>
       <xsl:apply-templates select="html:thead"/>
       <xsl:apply-templates select="html:tfoot"/>
       <xsl:choose>
       <xsl:when test="html:tbody">
           <xsl:apply-templates select="html:tbody"/>
       </xsl:when>
       <xsl:otherwise>
           <fo:table-body>
                <xsl:apply-templates select="html:tr"/>
           </fo:table-body>
       </xsl:otherwise>
       </xsl:choose>
   </fo:table>
<!--
   </fo:table-and-caption>
-->
   </xsl:template>

  
   <xsl:template   match="html:caption">
      <fo:block><xsl:apply-templates /></fo:block>
   </xsl:template>

  
   <xsl:template   match="html:thead">
   <fo:table-header>
      <xsl:apply-templates select="html:tr"/>
   </fo:table-header>
   </xsl:template>

  
   <xsl:template   match="html:tfoot">
   <fo:table-footer>
      <xsl:apply-templates select="html:tr"/>
   </fo:table-footer>
   </xsl:template>

  
   <xsl:template   match="html:tbody">
   <fo:table-body>
      <xsl:apply-templates select="html:tr"/>
   </fo:table-body>
   </xsl:template>

   <xsl:template   match="html:colgroup">
     <xsl:choose>
      <xsl:when test="html:col">
        <xsl:apply-templates select="html:col"/>
      </xsl:when>
      <xsl:otherwise>    <xsl:variable name="width">
    <xsl:choose>
      <xsl:when test="@width"><xsl:value-of select="@width" /></xsl:when>
      <xsl:otherwise></xsl:otherwise>
    </xsl:choose>
    </xsl:variable>
    <xsl:variable name="align">
    <xsl:choose>
      <xsl:when test="@align"><xsl:value-of select="@align" /></xsl:when>
      <xsl:otherwise>inherit</xsl:otherwise>
    </xsl:choose>
    </xsl:variable>
    <xsl:variable name="span">
    <xsl:choose>
      <xsl:when test="@span"><xsl:value-of select="@span" /></xsl:when>
      <xsl:otherwise>1</xsl:otherwise>
    </xsl:choose>
    </xsl:variable>
    <xsl:call-template name="make-column">
    <xsl:with-param name="width" select="$width" />
    <xsl:with-param name="align" select="$align" />
    <xsl:with-param name="span" select="$span" />
    </xsl:call-template>

      </xsl:otherwise>
     </xsl:choose>
   </xsl:template>
   
   
   <xsl:template   match="html:colgroup/html:col">
    <xsl:variable name="width">
    <xsl:choose>
      <xsl:when test="@width"><xsl:value-of select="@width" /></xsl:when>
      <xsl:otherwise>
        <xsl:if test="../@width" >
        <xsl:value-of select="../@width" />
        </xsl:if>
      </xsl:otherwise>
    </xsl:choose>
    </xsl:variable>
    <xsl:variable name="align">
    <xsl:choose>
      <xsl:when test="@align"><xsl:value-of select="@align" /></xsl:when>
        <xsl:otherwise>
          <xsl:choose>
            <xsl:when test="../@align" >
              <xsl:value-of select="../@align" />
            </xsl:when>
            <xsl:otherwise>1</xsl:otherwise>
          </xsl:choose>
       </xsl:otherwise>
    </xsl:choose>
    </xsl:variable>
    <xsl:variable name="span">
    <xsl:choose>
      <xsl:when test="@span"><xsl:value-of select="@span" /></xsl:when>
      <xsl:otherwise>1</xsl:otherwise>
    </xsl:choose>
    </xsl:variable>
    <xsl:call-template name="make-column">
    <xsl:with-param name="width" select="$width" />
    <xsl:with-param name="align" select="$align" />
    <xsl:with-param name="span" select="$span" />
    </xsl:call-template>

   </xsl:template>
   
  <xsl:template match="html:table/html:col">
    <xsl:variable name="width">
    <xsl:choose>
      <xsl:when test="@width"><xsl:value-of select="@width" /></xsl:when>
      <xsl:otherwise></xsl:otherwise>
    </xsl:choose>
    </xsl:variable>
    <xsl:variable name="align">
    <xsl:choose>
      <xsl:when test="@align"><xsl:value-of select="@align" /></xsl:when>
      <xsl:otherwise>inherit</xsl:otherwise>
    </xsl:choose>
    </xsl:variable>
    <xsl:variable name="span">
    <xsl:choose>
      <xsl:when test="@span"><xsl:value-of select="@span" /></xsl:when>
      <xsl:otherwise>1</xsl:otherwise>
    </xsl:choose>
    </xsl:variable>
    <xsl:call-template name="make-column">
    <xsl:with-param name="width" select="$width" />
    <xsl:with-param name="align" select="$align" />
    <xsl:with-param name="span" select="$span" />
    </xsl:call-template>
  </xsl:template>
  
  <xsl:template name="make-column">
  <xsl:param name="width" />
  <xsl:param name="align" />
  <xsl:param name="span" />
  <fo:table-column>
  <xsl:attribute name = "column-width">
     <xsl:value-of select="$width" />
  </xsl:attribute>
   <xsl:attribute name = "number-columns-repeated">
  <xsl:value-of select="$span" />
  </xsl:attribute>
  <xsl:attribute name="text-align">
      <xsl:choose>
      <xsl:when test="$align = 'left'">left</xsl:when>
      <xsl:when test="$align = 'center'">center</xsl:when>
      <xsl:when test="$align = 'right'">right</xsl:when>
      <xsl:otherwise>start</xsl:otherwise>
      </xsl:choose>
    </xsl:attribute>
  </fo:table-column>
  </xsl:template>

  <xsl:template match="html:tr">
    <fo:table-row keep-together="always"><xsl:apply-templates select="html:th | html:td"/></fo:table-row>
  </xsl:template>

  <xsl:template match="html:th">
     <fo:table-cell ><xsl:call-template name="make-cell" /></fo:table-cell>
  </xsl:template>

<!--  
  <xsl:template match="html:td">
    <fo:table-cell>
        <xsl:if test="../../@border">
           <xsl:attribute name="border-top-style" value="solid"/>
           <xsl:attribute name="border-top-width" value="{../../@border}pt"/>
           <xsl:attribute name="border-left-style" value="solid"/>
           <xsl:attribute name="border-left-width" value="4pt"/>
           <xsl:attribute name="border-right-style" value="solid"/>
           <xsl:attribute name="border-right-width" value="{@border}pt"/>
           <xsl:attribute name="border-bottom-style" value="solid"/>
           <xsl:attribute name="border-bottom-width" value="{@border}pt"/>
        </xsl:if>
        <xsl:call-template name="make-cell" />
    </fo:table-cell>
  </xsl:template>
 -->

  <xsl:template match="html:td">
    <fo:table-cell 
        border-top-style="solid" border-top-width="0.3pt"
        border-left-style="solid" border-left-width="0.3pt"
        border-right-style="solid" border-right-width="0.3pt"
        border-bottom-style="solid" border-bottom-width="0.3pt"
        padding-top="2pt" padding-left="2pt" padding-right="2pt" padding-bottom="2pt"
        >
        <xsl:call-template name="make-cell" />
    </fo:table-cell>
  </xsl:template>
 
 <xsl:template name="make-cell">

    <xsl:attribute name="text-align">from-table-column(text-align)</xsl:attribute>

   <xsl:if test="@colspan">
      <xsl:attribute name="number-columns-spanned">
        <xsl:value-of select="@colspan"/>
      </xsl:attribute>
   </xsl:if>
   <xsl:if test="@rowspan">
      <xsl:attribute name="number-rows-spanned">
        <xsl:value-of select="@rowspan"/>
      </xsl:attribute>
   </xsl:if>
   <xsl:if test="@align">
      <xsl:attribute name="text-align">
        <xsl:choose>
        <xsl:when test="@align = 'left'">left</xsl:when>
        <xsl:when test="@align = 'center'">center</xsl:when>
        <xsl:when test="@align = 'right'">right</xsl:when>
        <xsl:otherwise>inherit</xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>
   </xsl:if>
   <fo:block text-align="inherit">
   <xsl:apply-templates />
   </fo:block>
  </xsl:template>

 

</xsl:stylesheet>